#pragma once

#if defined(_WIN32)
#define QUBFITS_DLLEXPORT __declspec(dllexport)
#define QUBFITS_DLLIMPORT __declspec(dllimport)
#else
#define QUBFITS_DLLEXPORT
#define QUBFITS_DLLIMPORT
#define __stdcall
#define __cdecl
#endif

#if !defined(_QUBFITS_LIB_)
	#define QUBFITS_API QUBFITS_DLLIMPORT
#else
	#define QUBFITS_API QUBFITS_DLLEXPORT
#endif



extern "C" QUBFITS_API int    __stdcall initqubfits();



class IQUB_EvtListener
#if defined (_WIN32)
 : public IUnknown
	// ok ms and borland forced me to inherit from iunknown
	// but this is official notice i will ignore those 3 methods.
	// please implement them with an empty body, and never call them.
#endif
{
public:
  virtual void __cdecl QUBEvt(const char *name, void *sender) = 0;

  virtual ~IQUB_EvtListener() {}
};

class QUBFITS_API IQUB_Evts
{
public:
  virtual void __cdecl Free() = 0;
  virtual void  __cdecl add(const char *name, IQUB_EvtListener *rcvr) = 0;
  virtual void  __cdecl del(const char *name, IQUB_EvtListener *rcvr) = 0;
  virtual void  __cdecl send(const char *name, void *sender) = 0;

  virtual ~IQUB_Evts();
};

class QUBFITS_API QUB_Evts : public IQUB_Evts
{
public:
  QUB_Evts();
  virtual ~QUB_Evts();
  void __cdecl Free();
  void __cdecl add(const char *name, IQUB_EvtListener *rcvr);
  void __cdecl del(const char *name, IQUB_EvtListener *rcvr);
  void __cdecl send(const char *name, void *sender);
 private:
  void *listeners;
};

extern "C" QUBFITS_API IQUB_Evts *    __stdcall QUB_Evts_Create();


typedef struct
{
	int n;
	double *xx, *yy, *ww;
	int nv;
	char **vname;
	double **vvv;
} QF_Data;


// curve and fitter maintain complete state in a qubtree, always up-to-date


class IQUB_Curve
{
public:
  virtual void __cdecl Free() = 0;
  virtual IQUB_Evts * __cdecl Events() = 0;
  // Curve CompCount
  // Curve Eqn
  // Curve Params
  // Curve ParamVal
  // Curve ParamConst
  // Curve Msg
  // Curve Err
  virtual void __cdecl setExtraVars(char **names, int count) = 0;
  virtual const char * __cdecl getName() = 0;
  virtual const char * __cdecl getEqn() = 0;
  virtual void __cdecl setEqn(const char *x) = 0;
  virtual int __cdecl canSetEqn() = 0;
  virtual int __cdecl getComponentCount() = 0;
  virtual void __cdecl setComponentCount(int x) = 0;
  virtual int __cdecl getParamCount() = 0;
  virtual const char * __cdecl getParamName(int ix) = 0; // valid until you call this or another curve method
  virtual double __cdecl getParamVal(int ix) = 0;
  virtual void __cdecl setParamVal(int ix, double val, int sendEvent) = 0;
  virtual int __cdecl getParamConst(int ix) = 0;
  virtual void __cdecl setParamConst(int ix, int isConst) = 0;
  virtual void __cdecl initParams(double *xx, double *yy, int n) = 0;
  virtual void __cdecl evalArrayAll(QF_Data *data, double *yy) = 0;
  virtual void __cdecl evalArrayOne(QF_Data *data, double *yy, int component) = 0;
  virtual int __cdecl hasDeriv() = 0;
  virtual void __cdecl evalDerivArray(QF_Data *data, double *dpar) = 0;
  virtual const char * __cdecl getMessage() = 0; // if directed by event
  virtual QTR_Impl* __cdecl getState() = 0;
  virtual void __cdecl updateState(QTR_Impl* state) = 0;

  virtual ~IQUB_Curve() {}
};

typedef IQUB_Curve* (__stdcall * QUB_CurveFactory)();

QUBFITS_API IQUB_Curve* __stdcall CreateNullCurve();

extern "C" QUBFITS_API void __stdcall RegisterCurve(const char *name, QUB_CurveFactory factory);
extern "C" QUBFITS_API IQUB_Curve* __stdcall CreateCurve(const char *name);
extern "C" QUBFITS_API QTR_Impl* __stdcall ListCurves();
// Curves
// {
//     Curve = Exponential
//     ...

class IQUB_Fitter
{
public:
	virtual void __cdecl Free() = 0;
	virtual IQUB_Evts * __cdecl Events() = 0;
	// Fitter ParamVal
	// Fitter Iter
	// Fitter Msg
	// Fitter Err
	virtual const char * __cdecl getName() = 0;
	virtual int __cdecl getParamCount() = 0;
	virtual const char * __cdecl getParamName(int ix) = 0;
	virtual double __cdecl getParamVal(int ix) = 0;
	virtual void __cdecl setParamVal(int ix, double val) = 0;
	virtual int __cdecl Fit(IQUB_Curve* curve, QF_Data **data, int ndata) = 0;
	virtual void __cdecl Stop() = 0; // from another thread or inside event handler
	virtual int __cdecl getIterations() = 0;
	virtual double __cdecl getParamErr(int ix) = 0;
	virtual double * __cdecl getCurve() = 0;
	virtual const char * __cdecl getMessage() = 0; // if directed by event
	virtual QTR_Impl* __cdecl getState() = 0;
	virtual void __cdecl updateState(QTR_Impl* state) = 0;

  virtual ~IQUB_Fitter() {}
};

typedef IQUB_Fitter* (__stdcall * QUB_FitterFactory)();

QUBFITS_API IQUB_Fitter* __stdcall CreateNullFitter();

extern "C" QUBFITS_API void __stdcall RegisterFitter(const char *name, QUB_FitterFactory factory);
extern "C" QUBFITS_API IQUB_Fitter* __stdcall CreateFitter(const char *name);
extern "C" QUBFITS_API QTR_Impl* __stdcall ListFitters();

// Fitters
// {
//     Fitter = LMfit
//     ...

extern "C" QUBFITS_API double __stdcall ChiSqrUncertainty(IQUB_Curve* curve, QF_Data *data, double *ff, int ipar, double deltaPct/*0<d<1*/);


// in filterdatad.cpp
extern "C" QUBFITS_API int __stdcall 
  filterdatad(double *data, double *fdata, int ndata, double dt, double freq);

